package nckd.base.helper;

import kd.bos.exception.BosErrorCode;
import kd.bos.exception.ErrorCode;
import kd.bos.exception.KDException;
import kd.bos.fileservice.FileService;
import kd.bos.fileservice.enums.PreviewParams;
import kd.bos.fileservice.impl.*;
import kd.bos.fileservice.multiserver.Dispatcher;
import kd.bos.fileservice.multiserver.DispatcherFactory;
import kd.bos.fileservice.path.DecodeFileFactory;
import kd.bos.fileservice.preview.PreviewService;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.servicehelper.AttachmentServiceHelper;
import kd.bos.url.UrlService;
import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.*;


public class DefaultPreviewCusServiceImpl  implements PreviewService {
    private static Log logger = LogFactory.getLog(DefaultPreviewCusServiceImpl.class);
    private static final String MAGIC_STRING = "frSheet";
    private static final String COOKIE = "Cookie";
    private static final String BOS_FILESERVICE_SDK = "bos-fileservice-sdk";
    private static final String PREVIEW_CACHE_REFRESH = "filepreview.refresh.enable";
    private static final String TEMP_DIR = System.getProperty("java.io.tmpdir");
    private boolean isPreviewcacheEnable;
    private boolean isPreviewcacheRefresh;
    private int uploadTimeout;
    private FileService fileService;
    private YunHomeService yunHomeService;
    private Dispatcher fileServerDispatcher;
    private Dispatcher previewServerDispatcher;

    private static final String rootUrl = "http://127.0.0.1:8012/onlinePreview?url=";
    private final Base64.Encoder b64encoder = Base64.getEncoder();

    private static final String kkViewSupport = "doc,docx,xls,xlsx,ppt,pptx"; // 可查看官网补充

    public DefaultPreviewCusServiceImpl() {
    }

    public void init(FileService fileService) {
        logger.info("DefaultPreviewCusServiceImpl：" + fileService.toString());
        this.fileService = fileService;
//        this.isPreviewcacheEnable = AttachmentFileService.isPreviewcacheEnable() && fileService instanceof AttachmentFileService;
//        this.isPreviewcacheRefresh = Boolean.parseBoolean(System.getProperty("filepreview.refresh.enable"));
//        this.uploadTimeout = AttachmentFileService.getUploadTimeout();
//        this.fileServerDispatcher = DispatcherFactory.build("attachmentServer.url", BosErrorCode.fileServerNotConfigured);
//        this.previewServerDispatcher = DispatcherFactory.build("yunpan.previewUrl", BosErrorCode.previewFailed);
//        this.yunHomeService = new YunHomeService(fileService);
    }

    /**
     * 文件流预览模式
     * 系统参数预览方式可选择 默认方式 / 自定义
     */
    @Override
    public Map<String, Object> preview(String fileName, String path, String userAgent) {
        return null;
    }

    @Override
    public void removePreview(String id) {
    }

    /**
     * 文件流预览模式 - 临时文件预览
     * 系统参数预览方式选择 云之家
     */
    @Override
    public Map<String, Object> previewFromCache(String fileName, String path, String userAgent, InputStream cacheIn) {
        return null;
    }

    /**
     * WPS预览模式
     * 如返回第三方预览url
     */
    @Override
    public Map<String, Object> previewWPS(String fileName, String path, String userAgent, Map<String, String> config) {
        logger.info("testpreviewWPS...");
        Map<String, Object> result = new HashMap<>();
        String ext = getExt(fileName);
        if(this.isKkViewSupport(ext)){
            //1.需要转换的返回kkview的预览url
            if (fileName.contains("\\")) { // 处理轻量级服务文件名
                fileName = fileName.substring(fileName.lastIndexOf("\\"));
                fileName = fileName.replaceAll("\\\\", "");
            }
            String downloadUrl = this.getDownloadUrl(path);
            String url = getKkviewUrl(downloadUrl, fileName);
            result.put(PreviewParams.STATUS.getEnumName(), PreviewParams.URL.getEnumName());
            result.put(PreviewParams.RESULT.getEnumName(), url);
            return result;
        }

        InputStream fileStream = null;
        try {
            fileStream = fileService.getInputStream(path);

//            fileStream = FileSECUtils.processFileWithSEC(path, fileStream);

            //2.不需要转换的直接返回原来流 平台默认的预览
            result.put(PreviewParams.STATUS.getEnumName(), PreviewParams.NOT_NEED_CHANGE.getEnumName());
            result.put(PreviewParams.RESULT.getEnumName(), fileStream);
        } catch (Exception e) {
            String error = e.getMessage();
            // 3.失败返回格式
            result.put(PreviewParams.STATUS.getEnumName(), PreviewParams.ERROR.getEnumName());
            result.put(PreviewParams.DESCRIPTION.getEnumName(), error);
        } finally {
        }

        return result;
    }

    /**
     * wps预览模式 - 临时文件url预览
     * 如返回第三方预览url
     */
    @Override
    public Map<String, Object> previewFromCacheWPS(String fileName, String path, String userAgent, InputStream cacheIn, Map<String, String> config) {
        logger.info("previewFromCacheWPS...");
        InputStream fileStream = null;
        Map<String, Object> result = new HashMap<>();
        try {
            // 文件加密需要解码文件流
            fileStream = DecodeFileFactory.getDecodeService().getDecodeFileStream(cacheIn);
            String ext = getExt(fileName);
            if(this.isKkViewSupport(ext)) {
                //1、需要转换的返回转换后的流 kkview的预览
                String downloadUrl = this.getDownloadUrl(path);
                String url = getKkviewUrl(downloadUrl, fileName);
                result.put(PreviewParams.STATUS.getEnumName(), PreviewParams.URL.getEnumName());
                result.put(PreviewParams.RESULT.getEnumName(), url);
                return result;
            }
            //2、不需要转换的直接返回原来流 平台默认预览

//            fileStream = FileSECUtils.processFileWithSEC(path, fileStream);

            result.put(PreviewParams.STATUS.getEnumName(), PreviewParams.NOT_NEED_CHANGE.getEnumName());
            result.put(PreviewParams.RESULT.getEnumName(), fileStream);
        } finally {
        }

        return result;
    }

    @Override
    public void deletePreviewCacheWps(String url, Map<String, String> config) {
    }

    @Override
    public void deletePreviewWps(String path, Map<String, String> config) {
    }

    public String getExt(String fileName){
        String ext;
        String[] extTmp = fileName.split("\\.");
        ext = extTmp[extTmp.length-1];
        return ext;
    }

    private String getKkviewUrl(String url, String filename){
        // url构造见KkFileView官方文档 - 注意KkFileView会使用文件名缓存预览文件
        url += "&fullfilename=" + filename.replaceAll("[\\\\/]", "");
        url += "&officePreviewType=html"; // image,pdf,html
        String encodeUrl = b64encoder.encodeToString(url.getBytes(StandardCharsets.UTF_8));
        return rootUrl + encodeUrl;
    }

    private boolean isKkViewSupport(String ext) {
        ext = ext.replaceAll("\\.", "");
        return kkViewSupport.contains(ext);
    }

    private String getDownloadUrl(String path){
        String downloadUrl;
        if (path.contains("tempfile/download.do")) { // 临时文件为url链接
            downloadUrl = path.contains("kdedcba") ? path : AttachmentServiceHelper.getEncreptURL(path);
            return downloadUrl;
        }
        try { // 轻量级文件服务为本地文件地址
            path = URLEncoder.encode(path, "utf-8");
        } catch (UnsupportedEncodingException e) {
            throw new KDException(new ErrorCode("previewErr", "url encode err"), e);
        }
        downloadUrl = UrlService.getAttachmentDownloadUrl(path);
        downloadUrl = AttachmentServiceHelper.getEncreptURL(downloadUrl); // 追加临时免登授权
        // 开启csrf校验时，需要增加参数
        return downloadUrl;
    }
}
